#!/usr/bin/env python
"""
This module provides a `column controller' for a CTreeInsert.
NB:  A CTreeInsert is just a GtkCTree with a few added methods; see
CTreeInsert.py for more info.
      
The column controller provides these behaviors:
    o Left-click on column button sorts the tree on that column.
    o Right-click ... pops up a menu of column options.

If a column is inserted when there is no more space for columns,
raise an AssertionError
"""
import gtk, GDK, GtkExtra
from ColumnPopupMenu import ColumnPopupMenu

# XXX FRAGILE, BOGUS CODE FOLLOWS
# There's no easy way to find out the true size of a column such that:
#     c0 = get_column_width(i)
#     set_column_width(i, c0)
#     cf = get_column_width(i)
#     assert (c0 == cf)
# The only access method I've found involves asking for the width of
# the window in which each column-header button is drawn.  But that
# width encompasses the button *and* any cell padding imposed by the
# GtkCList widget.
# The cell padding information appears not to be available through
# the gtkclist API.
# The padding sizes *are* hardwired in gtkclist.c.  So my kludgey
# workaround is to copy the values (as of Gtk 1.2.3) here:
CELL_SPACING = 1
COLUMN_INSET = 3

class ColumnController:
    """This is a column controller for MeshTrix instances.
    See the module docstring for more details."""
    # For helping compute column widths.
    _decorWidth = CELL_SPACING + (2 * COLUMN_INSET)
    
    def __init__(self, tree):
        """Initialize a new instance.
        `tree' specifies the MeshTrix instance which self controls."""
        self.tree = tree

        self.num_columns = self.tree["n_columns"]

        self.connect_buttons()
        self.menu = ColumnPopupMenu(self.tree,
                                    set_column_visibility_cb=self.set_column_visibility_cb,
                                    insert_cb=self.insert_cb)
        
    def connect_buttons(self):
        """Wire up the buttons in self's tree."""
        for i in range(self.tree["n_columns"]):
            # Wire up all of the buttons, even if the column isn't
            # in use.  We'll try to ensure unused columns are
            # invisible.
            button = self.tree.get_column_widget(i)["parent"]
            button.connect("clicked", self.sort_column_cb, i)
            button.connect("button_press_event", self.button_pressed_cb, i)

    def insert_cb(self, widget):
        """insert a column in to the widget"""
        assert self.tree.used_columns < self.num_columns, \
               "can't insert (ran out of columns)"
        try:
            self.tree.freeze()
            # grab an unused column and make it visible
            self.tree.set_column_visibility(self.tree.used_columns, 1)
            self.tree.used_columns = self.tree.used_columns + 1
        finally:
            self.tree.thaw()
        
    def sort_column_cb(self, widget, i):
        """Callback invoked when the tree is to be sorted on column i."""
        self.sort_column(i)

    def sort_column(self, i):
        """Sort the tree using i as the primary sort column."""
        self.tree.set_sort_column(i)
        # Ugh.  This does a lexical sort only.  Need some way to
        # do generalized comparison-sort...
        self.tree.sort()

    def button_pressed_cb(self, widget, event, i):
        """Callback invoked when a mouse button is pressed on column
        header `i'."""
        # Popup the menu only if this was a right-button click.
        # (Assumes button 3 is always the `popup' button --
        # pardon my dexter-chauvinism.)
        if ((event.type == GDK.BUTTON_PRESS) and (event.button == 3)):
            self.popup_menu(event, i)
            return gtk.TRUE
        return gtk.FALSE

    def popup_menu(self, event, i_column):
        """[Build and] popup a menu for the specified column.
        `event' is the event triggering the popup."""
        self.menu.popup(event, i_column)

    def set_column_visibility_cb(self, widget, i, show):
        """Callback invoked to make column `i' visible/hidden."""
        self.tree.set_column_visibility(i, show)

    def print_column_titles(self):
        """DEBUG:  Print the current column titles to stdout."""
        titles = []
        for i in range(self.num_columns):
            titles.append(self.tree.get_column_title(i))
        print titles
        
    def column_widths(self):
        """Get the widths of self's controlled columns."""
        # XXX FUDGE!
        # See the XXX comments near the top of this module for
        # more info.  The short form is that I need to fudge
        # the returned width to account for things like
        # CList cell spacing, button border width, and
        # other decorations.

        tree_width, tree_height = self.tree.get_child_requisition()

        result = []
        for i in range(self.num_columns):
            widthNvis = self.get_column_width_and_visibility(i)
            result.append(widthNvis)

        return result

    def get_column_width_and_visibility(self, i):
        """Try to get the width and visibility of column i."""
        button = self.tree.get_column_widget(i)["parent"]
        # This approach sorta works for some pygtk < 0.6.1:
        width = 0
        window = button.get_window()
        if window:
            width = window.width

##         # And this sorta works for pygtk > 0.6.1:
##         width = button.get_allocation()[2]

        # What I really need is for pygtk to expose self.tree.column[].
        
        width = width - self._decorWidth
        return width, button["visible"]

def main():
    """Module mainline (for standalone execution)"""
    import gtk

    class Tree(gtk.GtkCTree):
	def __init__(self, cols=1, tree_col=0, titles=None, _obj=None):
                import _gtk

		if _obj: self._o = _obj; return
		if titles == None:
			self._o = _gtk.gtk_ctree_new(cols, tree_col)
		else:
			self._o = _gtk.gtk_ctree_new_with_titles(cols,
								 tree_col,
								 titles)
                self.used_columns = cols

        def has_columns(self):
            # no available columns
            return 0
        
        def column_in_use(self, i):
            title = self.get_column_title(i)
            return (title != "")

        def fill_tree(self, parent=None, depth=0):
            """Fill self with dummy nodes."""
            numCols = self["n_columns"]
            if depth == 0:
                self.numNodes = 0
                
            is_leaf = (depth >= 0)
            for i in range(1):
                # Make sure the label list has the same number of entries as
                # the list has columns.
                label = []
                for j in range(numCols):
                    title = self.get_column_title(j)
                    label.append("%s %s %s %s" % (title, depth, i, j))
                node = self.insert_node(parent, None, label, is_leaf=is_leaf)
                self.numNodes = self.numNodes + 1
                if is_leaf:
                    self.node_set_row_data(node, (label, depth, i, parent))
                else:
                    self.fill_tree(node, depth+1)

        def all_nodes(self):
            result = self.base_nodes()
            index = 0
            while 1:
                new_children = []
                next_index = len(result)
                for node in result[index:]:
                    new_children = new_children + node.children
                if not new_children:
                    break
                result = result + new_children
                index = next_index

            return result

    win = gtk.GtkWindow()
    
    titles=["Column 1", "Column 2", "", "Column 4", "Column 5", "", "Column 7"]
    tree = Tree(cols=len(titles), titles=titles)

    # Give the tree some bogus content.
    tree.fill_tree()
    tree.columns_autosize()

    # Attach a column controller.
    controller = ColumnController(tree)

    win.add(tree)
    tree.show()

    win.connect("delete_event", gtk.mainquit)
    win.set_title("TestColumnController -- %d Nodes" %
                  len(tree.all_nodes()))
    win.show()
    gtk.mainloop()


if __name__ == "__main__":
    main()
