#!/usr/bin/env python
"""
This module provides the base class for the PHLM viewer's row popup menu.
The popup menu has callback hooks for each of the menu items.

Copyright 1999 Bioreason, Inc.
$Id: ColumnRowPopup.py,v 1.3 1999/07/01 22:10:07 brian Exp $
Mitch Chapman
modified by Brian Kelley
"""

__version__ = "$Revision: 1.3 $"

import gtk
import GtkExtra, GDK

class ColumnRowPopup:
    """This is the base class for the PHLM viewer's row popup menu.
    See the module docstring for more info."""
    def __init__(self, tree):
        """Initialize a new instance.
        `tree' is the PHLM viewer tree on which to install the popup."""
        self.tree = tree
        self.tree.connect("button_press_event", self.button_pressed_cb)
	self.menuitems = []
	self.menu_sensitivity = []
	self.buttons = []
##        self.build_menu()

    def add_menuitem(self, menu_location, action_call_back,
		     sensitivity_call_back=None):
	# add the menu item in the GtkExtra.MenuFactory format
	self.menuitems.append([menu_location, None, action_call_back])
	self.menu_sensitivity.append(sensitivity_call_back)
	
    def build_menu(self):
        """Build the actual menu contents."""
        factory = GtkExtra.MenuFactory(type=GtkExtra.MENU_FACTORY_MENU)
        
##         self.display_smallest_btn = factory.create("/Display Smallest", None,
##                                                    self.display_smallest_cb)
##         self.display_tested_btn = factory.create("/Display Tested", None,
##                                                  self.display_tested_cb)
	for menuitem in self.menuitems:
	    self.buttons.append(factory.create(menuitem[0], None, menuitem[2]))

        self.menu = factory.get_menu('')

    def button_pressed_cb(self, widget, event, *args):
        """Callback invoked when a button is pressed anywhere on
        self's tree."""
        # Popup the menu only on a button-3 (right mouse?) press.
        # This logic really ought to be pushed into a utility function;
        # it's used in other modules to identify right-mouse clicks.
        if ((event.type == GDK.BUTTON_PRESS) and (event.button == 3)):
            self.select_and_popup(event)
            return gtk.TRUE
        return gtk.FALSE

    def select_and_popup(self, event):
        """Adjust the tree's selection, as needed, and popup self's menu.
        If the button press event occurs on a row which is not currently
        selected, then the selection is changed to be *only* the row
        on which the button was pressed.  Otherwise the selection is
        unaltered."""
        info = self.tree.get_selection_info(event.x, event.y)
        # There may be no selection.
        if info:
            row, col = info
            self.adjust_selection(row, col)
            self.popup_menu(event, row, col)

    def adjust_selection(self, row, col):
        """Adjust the selection to ensure the specified row/col is selected.
        Strategy:  if the tree node at (row, col) is already selected, leave
        the selection as-is.  If it is not selected, then clear the current
        selection and select the node at (row, col)."""
##        selection = self.tree.selection

        node = self.tree.node_nth(row)
##        if node not in selection:
	self.tree.freeze()
	self.tree.unselect_all()
	self.tree.select(node)
	self.tree.thaw()
        
    def popup_menu(self, event, row, col):
        """Popup a menu at the specified event location.
        `row' and `col' tell which tree item is covered by the click."""
        self.update_menu()
        self.menu.popup(None, None, None, event.button, event.time)

    def update_menu(self):
        """Update the menu contents to reflect the current selection."""
        # Enable each menu item iff the selection contains a single
        # row.
	# assume all buttons are "off"
	for button in self.buttons:
	    button["sensitive"] = 0

	index = 0
	for callback in self.menu_sensitivity:
	    if callback and callback(self.tree):
		self.buttons[index]["sensitive"] = 1
	    index = index + 1	    

	    
##         sensitive = len(self.tree.selection) == 1
##         self.display_smallest_btn["sensitive"] = sensitive
##         self.display_tested_btn["sensitive"] = sensitive

    def display_smallest_cb(self, widget, *args):
        """Callback for the 'Display Smallest' menu item."""
        return

    def display_tested_cb(self, widget, *args):
        """Callback for the 'Display Tested' menu item."""
        return


def main():
    """Module mainline (for standalone execution)"""
    class Tree(gtk.GtkCTree):
        def fill_tree(self, parent=None, depth=0):
            """Fill self with dummy nodes."""
            numCols = self["n_columns"]
            is_leaf = (depth >= 3)
            for i in range(3):
                # Make sure the label list has the same number of entries as
                # the list has columns.
                label = []
                for j in range(numCols):
                    title = self.get_column_title(j)
                    label.append("%s %s %s %s" % (title, depth, i, j))
                node = self.insert_node(parent, None, label, is_leaf=is_leaf)
                if is_leaf:
                    self.node_set_row_data(node, (label, depth, i, parent))
                else:
                    self.fill_tree(node, depth+1)

    win = gtk.GtkWindow()
    # Good to know:  pygtk will dump core if you don't specify a
    # number of columns and corresponding column titles in its selector.
    tree = Tree(cols=1, titles=["Select Me"])
    tree.fill_tree()
    tree.set_selection_mode(gtk.SELECTION_MULTIPLE)
    win.add(tree)
    tree.show()
    
    def display_smallest(widget, **args):
	print "displaying row"

    def check_display_smallest(tree):
	return len(tree.selection) == 1

    t = ColumnRowPopup(tree)
    t.add_menuitem("/Display Smallest", display_smallest,
		   check_display_smallest)
    t.add_menuitem("/Display Smallest2", display_smallest,
		   check_display_smallest)
    t.build_menu()


    win.connect("delete_event", gtk.mainquit)
    win.show()
    gtk.mainloop()


if __name__ == "__main__":
    main()
