/*
 *    General purpose FIR filter routines
 *    Copyright (C) 1999  Jim A. Ledin (jim@ledin.com)
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *    The file filter.dat must #define N to be the number
 *    of filter coefficients and the array double H[N]
 *    that contains the coefficients
 */

#include "filter.dat"

static double prev_x[N];

/* Initialize the input history to a constant value */
void Initialize(double value)
{
    int i;
    for (i=0; i<N; i++)
        prev_x[i] = value;
}

/* Given an input sample, compute the filter output */
double Filter(double x)
{
    int i, j;
    double sum = 0;
    static int index = 0;

    prev_x[index++] = x;

    /* If past the end, wrap back to the beginning */
    if (index >= N)
        index = 0;
    
    /* Compute the filter output */
    for (i=0; i<N; i++)
    {
        j = index - i;
        if (j < 0)
            j += N;
        
        sum += H[i]*prev_x[j];
    }

    return sum;
}